package cn.flyrise.iot.protocol.agree.http;

import cn.flyrise.iot.protocol.config.ObjectMappers;
import cn.flyrise.iot.protocol.topic.TopicMessageCodec;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.netty.buffer.Unpooled;
import lombok.AllArgsConstructor;
import org.jetlinks.core.message.DeviceMessage;
import org.jetlinks.core.message.RepayableDeviceMessage;
import org.jetlinks.core.message.codec.*;
import org.jetlinks.core.message.codec.http.HttpExchangeMessage;
import org.jetlinks.core.message.codec.http.SimpleHttpResponseMessage;
import org.springframework.http.MediaType;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.Nonnull;

@AllArgsConstructor
public class PaiHttpDeviceMessageCodec implements DeviceMessageCodec {

    private final Transport transport;

    private final ObjectMapper mapper;

    public PaiHttpDeviceMessageCodec(Transport transport) {
        this.transport = transport;
        this.mapper = ObjectMappers.JSON_MAPPER;
    }

    public PaiHttpDeviceMessageCodec() {
        this(DefaultTransport.HTTP);
    }

    @Override
    public Transport getSupportTransport() {
        return transport;
    }

    /**
     * 解码
     * @param context
     * @return
     */
    @Nonnull
    @Override
    public Flux<DeviceMessage> decode(@Nonnull MessageDecodeContext context) {
        HttpExchangeMessage message = (HttpExchangeMessage) context.getMessage();

        // 消息解码
        return TopicMessageCodec
                .decode(mapper, TopicMessageCodec.removeProductPath(message.getPath()), message.payloadAsBytes())
                .switchIfEmpty(Mono.defer(() -> {
                    //未转换成功，响应404
                    return message.response(SimpleHttpResponseMessage
                            .builder()
                            .status(404)
                            .contentType(MediaType.APPLICATION_JSON)
                            .payload(Unpooled.wrappedBuffer("{\"success\":false}".getBytes()))
                            .build()).then(Mono.empty());
                }))
                .flatMap(msg -> {
                    //响应成功
                    return message.response(SimpleHttpResponseMessage
                            .builder()
                            .status(200)
                            .contentType(MediaType.APPLICATION_JSON)
                            .payload(Unpooled.wrappedBuffer("{\"success\":true}".getBytes()))
                            .build())
                            .thenReturn(msg);
                });
    }

    /**
     * 编码
     * @param context
     * @return
     */
    public Mono<EncodedMessage> encode(MessageEncodeContext context) {

        return context
            .reply(((RepayableDeviceMessage<?>) context.getMessage()).newReply().success())
            .then(Mono.empty());
    }


}
