package cn.flyrise.iot.protocol.agree.mqtt;

import org.jetlinks.core.Value;
import org.jetlinks.core.defaults.Authenticator;
import org.jetlinks.core.device.*;
import reactor.core.publisher.Mono;

import javax.annotation.Nonnull;

/**
 * MQTT 认证授权
 * @author zhangqiang
 * @since 0.1
 */
public class PaiMqttAuthenticator implements Authenticator {

    /**
     *  在MQTT服务网关中指定了认证协议时,将调用此方法进行认证。
     *  注意: 认证通过后,需要设置设备ID.{@link AuthenticationResponse#success(String)}
     * @param request  认证请求
     * @param registry 设备注册中心
     * @return 认证结果
     */
    @Override
    public Mono<AuthenticationResponse> authenticate(@Nonnull AuthenticationRequest request, @Nonnull DeviceRegistry registry) {
        if (request instanceof MqttAuthenticationRequest) {
            MqttAuthenticationRequest mqtt = ((MqttAuthenticationRequest) request);

            return registry
                    .getDevice(mqtt.getClientId())
                    .flatMap(device -> authenticate(request, device));
        }
        return Mono.just(AuthenticationResponse.error(400, "不支持的授权类型:" + request));

    }

    /**
     * 对指定对设备进行认证
     *
     * @param request 认证请求
     * @param deviceOperation  设备
     * @return 认证结果
     */
    @Override
    public Mono<AuthenticationResponse> authenticate(@Nonnull AuthenticationRequest request, @Nonnull DeviceOperator deviceOperation) {
        if (request instanceof MqttAuthenticationRequest) {
            MqttAuthenticationRequest mqtt = ((MqttAuthenticationRequest) request);

            if(!mqtt.getClientId().equals(deviceOperation.getDeviceId())) return Mono.just(AuthenticationResponse.error(400, "客户端clientId与设备deviceId不一致"));
            return deviceOperation.getConfigs("username", "password")
                    .flatMap(values -> {
                        String username = values.getValue("username").map(Value::asString).orElse(null);
                        String password = values.getValue("password").map(Value::asString).orElse(null);
                        if (mqtt.getUsername().equals(username) && mqtt
                                .getPassword()
                                .equals(password)) {
                            return Mono.just(AuthenticationResponse.success());
                        } else {
                            return Mono.just(AuthenticationResponse.error(400, "账号或密码错误"));
                        }

                    });
        }
        return Mono.just(AuthenticationResponse.error(400, "不支持的授权类型:" + request));
    }
}
