package cn.flyrise.iot.protocol.agree.websocket;

import cn.flyrise.iot.protocol.config.ObjectMappers;
import cn.flyrise.iot.protocol.topic.TopicMessageCodec;
import cn.flyrise.iot.protocol.topic.TopicPayload;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.netty.buffer.Unpooled;
import org.jetlinks.core.message.DeviceMessage;
import org.jetlinks.core.message.DisconnectDeviceMessage;
import org.jetlinks.core.message.Message;
import org.jetlinks.core.message.codec.*;
import org.jetlinks.core.message.codec.http.websocket.DefaultWebSocketMessage;
import org.jetlinks.core.message.codec.http.websocket.WebSocketMessage;
import org.jetlinks.core.message.codec.http.websocket.WebSocketSession;
import org.jetlinks.core.message.codec.http.websocket.WebSocketSessionMessage;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.Nonnull;


public class PaiWebsocketDeviceMessageCodec implements DeviceMessageCodec {

    private final Transport transport;

    private final ObjectMapper mapper;

    public PaiWebsocketDeviceMessageCodec(Transport transport) {
        this.transport = transport;
        this.mapper = ObjectMappers.JSON_MAPPER;
    }

    public PaiWebsocketDeviceMessageCodec() {
        this(DefaultTransport.WebSocket);
    }

    @Override
    public Transport getSupportTransport() {
        return transport;
    }

    /**
     * 解码
     * @param context
     * @return
     */
    @Nonnull
    @Override
    public Flux<DeviceMessage> decode(@Nonnull MessageDecodeContext context) {
        WebSocketSessionMessage mqttMessage = (WebSocketSessionMessage) context.getMessage();
        WebSocketSession session = mqttMessage.getWebSocketSession();

        byte[] payload = mqttMessage.payloadAsBytes();

        return TopicMessageCodec
                .decode(mapper, TopicMessageCodec.removeProductPath(session.getUri()), payload)
                .switchIfEmpty(Mono.defer(() -> {
                    //未转换成功，响应404
                    return session
                            .send(session.textMessage("{\"status\":404}"))
                            .then(Mono.empty());
                }));
    }


    public Mono<EncodedMessage> encode(MessageEncodeContext context) {
        return Mono.defer(() -> {
            Message message = context.getMessage();

            if (message instanceof DisconnectDeviceMessage) {
                return ((ToDeviceMessageContext) context)
                        .disconnect()
                        .then(Mono.empty());
            }

            if (message instanceof DeviceMessage) {
                DeviceMessage deviceMessage = ((DeviceMessage) message);
                TopicPayload msg = TopicMessageCodec.encode(mapper, deviceMessage);
                if (null == msg) {
                    return Mono.empty();
                }
                JSONObject data = new JSONObject();
                data.put("topic", msg.getTopic());
                data.put("message", msg.getPayload());

                return Mono.just(DefaultWebSocketMessage.of(WebSocketMessage.Type.TEXT, Unpooled.wrappedBuffer(data.toJSONString().getBytes())));
            }
            return Mono.empty();

        });
    }


}
